==> scala-k8s-objects_sjs1_2.13-0.4.0-sources.jar: io/k8s/api/storage/v1/CSINode.scala <==
package io.k8s.api.storage.v1

import dev.hnaderi.k8s._
import dev.hnaderi.k8s.utils._

/** CSINode holds information about all CSI drivers installed on a node. CSI drivers do not need to create the CSINode object directly. As long as they use the node-driver-registrar sidecar container, the kubelet will automatically populate the CSINode object for the CSI driver as part of kubelet plugin registration. CSINode has the same name as a node. If the object is missing, it means either there are no CSI Drivers available on the node, or the Kubelet version is low enough that it doesn't create this object. CSINode has an OwnerReference that points to the corresponding node object. */
final case class CSINode(
  spec : io.k8s.api.storage.v1.CSINodeSpec,
  metadata : Option[io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta] = None
) extends KObject {
  protected val _resourceKind = ResourceKind("storage.k8s.io", "CSINode", "v1")


  /** Returns a new data with spec set to new value */
  def withSpec(value: io.k8s.api.storage.v1.CSINodeSpec) : CSINode = copy(spec = value)
  /** transforms spec to result of function */
  def mapSpec(f: io.k8s.api.storage.v1.CSINodeSpec => io.k8s.api.storage.v1.CSINodeSpec) : CSINode = copy(spec = f(spec))

  /** Returns a new data with metadata set to new value */
  def withMetadata(value: io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta) : CSINode = copy(metadata = Some(value))
  /** if metadata has a value, transforms to the result of function*/
  def mapMetadata(f: io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta => io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta) : CSINode = copy(metadata = metadata.map(f))

  override def foldTo[T : Builder] : T = CSINode.encoder.apply(this)
}

object CSINode {

    implicit val encoder : Encoder[io.k8s.api.storage.v1.CSINode] = new Encoder[io.k8s.api.storage.v1.CSINode] {
        def apply[T : Builder](o: io.k8s.api.storage.v1.CSINode) : T = {
          val obj = ObjectWriter[T]()
          obj
            .write("spec", o.spec)
            .write("metadata", o.metadata)
            .write("kind", o.kind)
            .write("apiVersion", o.apiVersion)
            .build
        }
    }

    implicit val decoder: Decoder[CSINode] = new Decoder[CSINode] {
      def apply[T : Reader](t: T): Either[String, CSINode] = for {
          obj <- ObjectReader(t)
          spec <- obj.read[io.k8s.api.storage.v1.CSINodeSpec]("spec")
          metadata <- obj.readOpt[io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta]("metadata")
      } yield CSINode (
          spec = spec,
          metadata = metadata
        )
    }
}

==> scala-k8s-objects_sjs1_3-0.3.0-sources.jar: io/k8s/api/storage/v1/CSINode.scala <==
package io.k8s.api.storage.v1

import dev.hnaderi.k8s._
import dev.hnaderi.k8s.utils._

/** CSINode holds information about all CSI drivers installed on a node. CSI drivers do not need to create the CSINode object directly. As long as they use the node-driver-registrar sidecar container, the kubelet will automatically populate the CSINode object for the CSI driver as part of kubelet plugin registration. CSINode has the same name as a node. If the object is missing, it means either there are no CSI Drivers available on the node, or the Kubelet version is low enough that it doesn't create this object. CSINode has an OwnerReference that points to the corresponding node object. */
final case class CSINode(
  spec : io.k8s.api.storage.v1.CSINodeSpec,
  metadata : Option[io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta] = None
) extends KObject {
  protected val _resourceKind = ResourceKind("storage.k8s.io", "CSINode", "v1")


  /** Returns a new data with spec set to new value */
  def withSpec(value: io.k8s.api.storage.v1.CSINodeSpec) : CSINode = copy(spec = value)
  /** transforms spec to result of function */
  def mapSpec(f: io.k8s.api.storage.v1.CSINodeSpec => io.k8s.api.storage.v1.CSINodeSpec) : CSINode = copy(spec = f(spec))

  /** Returns a new data with metadata set to new value */
  def withMetadata(value: io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta) : CSINode = copy(metadata = Some(value))
  /** if metadata has a value, transforms to the result of function*/
  def mapMetadata(f: io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta => io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta) : CSINode = copy(metadata = metadata.map(f))

  override def foldTo[T : Builder] : T = CSINode.encoder[T].apply(this)
}

object CSINode {

    implicit def encoder[T](implicit builder : Builder[T]) : Encoder[io.k8s.api.storage.v1.CSINode, T] = new Encoder[io.k8s.api.storage.v1.CSINode, T] {
        def apply(o: io.k8s.api.storage.v1.CSINode) : T = {
          val obj = ObjectWriter[T]()
          obj
            .write("spec", o.spec)
            .write("metadata", o.metadata)
            .write("kind", o.kind)
            .write("apiVersion", o.apiVersion)
            .build
        }
    }

    implicit def decoderOf[T : Reader] : Decoder[T, CSINode] = new Decoder[T, CSINode] {
      def apply(t: T): Either[String, CSINode] = for {
          obj <- ObjectReader(t)
          spec <- obj.read[io.k8s.api.storage.v1.CSINodeSpec]("spec")
          metadata <- obj.readOpt[io.k8s.apimachinery.pkg.apis.meta.v1.ObjectMeta]("metadata")
      } yield CSINode (
          spec = spec,
          metadata = metadata
        )
    }
}
